"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var React = _interopRequireWildcard(require("react"));

var _reactRedux = require("react-redux");

var _actions = require("redux-query/src/actions");

var _queryKey2 = require("redux-query/src/lib/query-key");

var _useConstCallback = _interopRequireDefault(require("../hooks/use-const-callback"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var normalizeToArray = function normalizeToArray(maybe) {
  return (Array.isArray(maybe) ? maybe : [maybe]).filter(Boolean);
};

var difference = function difference(a, b) {
  var bSet = new Set(b);
  return a.filter(function (x) {
    return !bSet.has(x);
  });
};

var getDiff = function getDiff(prevQueryConfigs, queryConfigs) {
  var prevQueryKeys = prevQueryConfigs.map(function (config) {
    return (0, _queryKey2.getQueryKey)(config);
  });
  var queryKeys = queryConfigs.map(function (config) {
    return (0, _queryKey2.getQueryKey)(config);
  });
  var queryConfigByQueryKey = queryKeys.reduce(function (accum, queryKey, i) {
    var queryConfig = queryConfigs[i];

    if (queryConfig) {
      accum.set(queryKey, queryConfig);
    }

    return accum;
  }, new Map());
  var cancelKeys = difference(prevQueryKeys, queryKeys).filter(Boolean);
  var requestKeys = difference(queryKeys, prevQueryKeys).filter(Boolean);
  var requestQueryConfigs = requestKeys.map(function (queryKey) {
    return queryConfigByQueryKey.get(queryKey);
  }).filter(Boolean);
  return {
    cancelKeys: cancelKeys,
    requestQueryConfigs: requestQueryConfigs
  };
};

var useMemoizedQueryConfigs = function useMemoizedQueryConfigs(mapPropsToConfigs, props, callback) {
  var queryConfigs = normalizeToArray(mapPropsToConfigs(props)).map(function (queryConfig) {
    var queryKey = (0, _queryKey2.getQueryKey)(queryConfig);

    if (queryKey) {
      return _objectSpread({}, queryConfig, {
        unstable_preDispatchCallback: function unstable_preDispatchCallback() {
          callback(queryKey);
        }
      });
    }
  }).filter(Boolean);

  var _React$useState = React.useState(queryConfigs),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      memoizedQueryConfigs = _React$useState2[0],
      setMemoizedQueryConfigs = _React$useState2[1];

  var previousQueryKeys = React.useRef(queryConfigs.map(_queryKey2.getQueryKey).filter(Boolean));
  React.useEffect(function () {
    var queryKeys = queryConfigs.map(_queryKey2.getQueryKey).filter(Boolean);

    if (queryKeys.length !== previousQueryKeys.current.length || queryKeys.some(function (queryKey, i) {
      return previousQueryKeys.current[i] !== queryKey;
    })) {
      previousQueryKeys.current = queryKeys;
      setMemoizedQueryConfigs(queryConfigs);
    }
  }, [queryConfigs]);
  return memoizedQueryConfigs;
};

var useMultiRequest = function useMultiRequest(mapPropsToConfigs, props) {
  var reduxDispatch = (0, _reactRedux.useDispatch)();
  var previousQueryConfigs = React.useRef([]);
  var pendingRequests = React.useRef(new Set());
  var dispatchRequestToRedux = (0, _useConstCallback["default"])(function (queryConfig) {
    var promise = reduxDispatch((0, _actions.requestAsync)(queryConfig));

    if (promise) {
      var _queryKey = (0, _queryKey2.getQueryKey)(queryConfig);

      if (_queryKey) {
        pendingRequests.current.add(_queryKey);
      }
    }
  });
  var dispatchCancelToRedux = (0, _useConstCallback["default"])(function (queryKey) {
    reduxDispatch((0, _actions.cancelQuery)(queryKey));
    pendingRequests.current["delete"](queryKey);
  });
  var queryConfigs = useMemoizedQueryConfigs(mapPropsToConfigs, props, function (queryKey) {
    pendingRequests.current["delete"](queryKey);
  });
  React.useEffect(function () {
    var _getDiff = getDiff(previousQueryConfigs.current, queryConfigs),
        cancelKeys = _getDiff.cancelKeys,
        requestQueryConfigs = _getDiff.requestQueryConfigs;

    requestQueryConfigs.forEach(dispatchRequestToRedux);
    cancelKeys.forEach(function (queryKey) {
      return dispatchCancelToRedux(queryKey);
    });
    previousQueryConfigs.current = queryConfigs;
    return function () {
      _toConsumableArray(pendingRequests.current).forEach(dispatchCancelToRedux);
    };
  }, [dispatchCancelToRedux, dispatchRequestToRedux, queryConfigs]);
  var forceRequest = React.useCallback(function () {
    queryConfigs.forEach(function (requestReduxAction) {
      dispatchRequestToRedux(_objectSpread({}, requestReduxAction, {
        force: true
      }));
    });
  }, [dispatchRequestToRedux, queryConfigs]);
  return forceRequest;
};

var connectRequest = function connectRequest(mapPropsToConfigs, options) {
  return function (WrappedComponent) {
    var _ref = options || {},
        _ref$pure = _ref.pure,
        pure = _ref$pure === void 0 ? true : _ref$pure,
        _ref$forwardRef = _ref.forwardRef,
        forwardRef = _ref$forwardRef === void 0 ? false : _ref$forwardRef;

    var ConnectRequestFunction = function ConnectRequestFunction(props) {
      var forceRequest = useMultiRequest(mapPropsToConfigs, props);
      return React.createElement(WrappedComponent, _extends({}, props, {
        forceRequest: forceRequest
      }));
    };

    var ConnectRequest = pure ? React.memo(ConnectRequestFunction) : ConnectRequestFunction;
    var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
    var displayName = "ConnectRequest(".concat(wrappedComponentName, ")");
    ConnectRequest.displayName = displayName;

    if (forwardRef) {
      var forwarded = React.forwardRef(function (props, ref) {
        return React.createElement(ConnectRequest, _extends({}, props, {
          forwardedRef: ref
        }));
      });
      forwarded.displayName = displayName;
      return (0, _hoistNonReactStatics["default"])(forwarded, WrappedComponent);
    }

    return (0, _hoistNonReactStatics["default"])(ConnectRequest, WrappedComponent);
  };
};

var _default = connectRequest;
exports["default"] = _default;